@@ -32,18 +32,21 @@
 
 #include <libempathy/empathy-contact-list.h>
 #include <libempathy/empathy-contact-manager.h>
-#include <libempathy/gossip-contact.h>
-#include <libempathy/gossip-debug.h>
-#include <libempathy/gossip-utils.h>
-#include <libempathy/gossip-conf.h>
+#include <libempathy/empathy-contact.h>
+#include <libempathy/empathy-tp-chat.h>
+#include <libempathy/empathy-debug.h>
+#include <libempathy/empathy-utils.h>
+#include <libempathy/empathy-conf.h>
 #include <libempathy/empathy-idle.h>
+#include <libempathy/empathy-filter.h>
 
 #include "empathy-status-icon.h"
 #include "empathy-contact-dialogs.h"
-#include "gossip-presence-chooser.h"
-#include "gossip-preferences.h"
-#include "gossip-ui-utils.h"
-#include "gossip-accounts-dialog.h"
+#include "empathy-presence-chooser.h"
+#include "empathy-preferences.h"
+#include "empathy-ui-utils.h"
+#include "empathy-accounts-dialog.h"
+#include "empathy-images.h"
 
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
@@ -54,24 +57,26 @@
 /* Number of ms to wait when blinking */
 #define BLINK_TIMEOUT 500
 
+typedef struct _StatusIconEvent StatusIconEvent;
+
 struct _EmpathyStatusIconPriv {
 	GtkStatusIcon         *icon;
 	EmpathyContactManager *manager;
+	EmpathyFilter         *text_filter;
 	EmpathyIdle           *idle;
+	MissionControl        *mc;
 	GList                 *events;
+	gboolean               showing_event_icon;
+	StatusIconEvent       *flash_state_event;
 	guint                  blink_timeout;
-	gboolean               showing_state_icon;
 
 	GtkWindow             *window;
-
 	GtkWidget             *popup_menu;
 	GtkWidget             *show_window_item;
 	GtkWidget             *message_item;
 	GtkWidget             *status_item;
 };
 
-typedef struct _StatusIconEvent StatusIconEvent;
-
 typedef void (*EventActivatedFunc) (StatusIconEvent *event);
 
 struct _StatusIconEvent {
@@ -85,9 +90,14 @@ struct _StatusIconEvent {
 static void       empathy_status_icon_class_init  (EmpathyStatusIconClass *klass);
 static void       empathy_status_icon_init        (EmpathyStatusIcon      *icon);
 static void       status_icon_finalize            (GObject                *object);
-static void       status_icon_idle_notify_cb      (EmpathyIdle            *idle,
-						   GParamSpec             *param,
+static void       status_icon_filter_new_channel  (EmpathyFilter          *filter,
+						   TpConn                 *tp_conn,
+						   TpChan                 *tp_chan,
 						   EmpathyStatusIcon      *icon);
+static void       status_icon_message_received_cb (EmpathyTpChat          *tp_chat,
+						   EmpathyMessage         *message,
+						   EmpathyStatusIcon      *icon);
+static void       status_icon_idle_notify_cb      (EmpathyStatusIcon      *icon);
 static void       status_icon_update_tooltip      (EmpathyStatusIcon      *icon);
 static void       status_icon_set_from_state      (EmpathyStatusIcon      *icon);
 static void       status_icon_toggle_visibility   (EmpathyStatusIcon      *icon);
@@ -108,10 +118,12 @@ static void       status_icon_quit_cb             (GtkWidget              *windo
 static void       status_icon_show_hide_window_cb (GtkWidget              *widget,
 						   EmpathyStatusIcon      *icon);
 static void       status_icon_local_pending_cb    (EmpathyContactManager  *manager,
-						   GossipContact          *contact,
+						   EmpathyContact          *contact,
 						   gchar                  *message,
 						   EmpathyStatusIcon      *icon);
 static void       status_icon_event_subscribe_cb  (StatusIconEvent        *event);
+static void       status_icon_event_flash_state_cb (StatusIconEvent       *event);
+static void       status_icon_event_msg_cb        (StatusIconEvent        *event);
 static StatusIconEvent * status_icon_event_new    (EmpathyStatusIcon      *icon,
 						   const gchar            *icon_name,
 						   const gchar            *message);
@@ -143,15 +155,22 @@ empathy_status_icon_init (EmpathyStatusIcon *icon)
 	priv->icon = gtk_status_icon_new ();
 	priv->idle = empathy_idle_new ();
 	priv->manager = empathy_contact_manager_new ();
-	priv->showing_state_icon = TRUE;
+	priv->mc = empathy_mission_control_new ();
+	priv->text_filter = empathy_filter_new ("org.gnome.Empathy.Chat",
+						"/org/freedesktop/Telepathy/Filter",
+						TP_IFACE_CHANNEL_TYPE_TEXT,
+						MC_FILTER_PRIORITY_DIALOG,
+						MC_FILTER_FLAG_INCOMING);
 
 	status_icon_create_menu (icon);
-	status_icon_set_from_state (icon);
-	status_icon_update_tooltip (icon);
+	status_icon_idle_notify_cb (icon);
 
-	g_signal_connect (priv->idle, "notify",
-			  G_CALLBACK (status_icon_idle_notify_cb),
+	g_signal_connect (priv->text_filter, "new-channel",
+			  G_CALLBACK (status_icon_filter_new_channel),
 			  icon);
+	g_signal_connect_swapped (priv->idle, "notify",
+				  G_CALLBACK (status_icon_idle_notify_cb),
+				  icon);
 	g_signal_connect (priv->icon, "activate",
 			  G_CALLBACK (status_icon_activate_cb),
 			  icon);
@@ -193,6 +212,7 @@ status_icon_finalize (GObject *object)
 	g_object_unref (priv->window);
 	g_object_unref (priv->idle);
 	g_object_unref (priv->manager);
+	g_object_unref (priv->mc);
 }
 
 EmpathyStatusIcon *
@@ -214,10 +234,10 @@ empathy_status_icon_new (GtkWindow *window)
 			  G_CALLBACK (status_icon_delete_event_cb),
 			  icon);
 
-	gossip_conf_get_bool (gossip_conf_get (),
-			      GOSSIP_PREFS_UI_MAIN_WINDOW_HIDDEN,
+	empathy_conf_get_bool (empathy_conf_get (),
+			      EMPATHY_PREFS_UI_MAIN_WINDOW_HIDDEN,
 			      &should_hide);
-	visible = gossip_window_get_is_visible (window);
+	visible = empathy_window_get_is_visible (window);
 
 	if ((!should_hide && !visible) || (should_hide && visible)) {
 		status_icon_toggle_visibility (icon);
@@ -227,15 +247,96 @@ empathy_status_icon_new (GtkWindow *window)
 }
 
 static void
-status_icon_idle_notify_cb (EmpathyIdle       *idle,
-			    GParamSpec        *param,
-			    EmpathyStatusIcon *icon)
+status_icon_filter_new_channel (EmpathyFilter     *filter,
+				TpConn            *tp_conn,
+				TpChan            *tp_chan,
+				EmpathyStatusIcon *icon)
 {
 	EmpathyStatusIconPriv *priv;
+	McAccount             *account;
+	EmpathyTpChat         *tp_chat;
+	GList                 *messages;
 
 	priv = GET_PRIV (icon);
 
-	if (priv->showing_state_icon) {
+	empathy_debug (DEBUG_DOMAIN, "New text channel to be filtered");
+
+	account = mission_control_get_account_for_connection (priv->mc, tp_conn, NULL);
+	tp_chat = empathy_tp_chat_new (account, tp_chan);
+	g_object_set_data (G_OBJECT (tp_chat), "filter", filter);
+	g_object_unref (account);
+
+	messages = empathy_tp_chat_get_pendings (tp_chat);
+	if (!messages) {
+		empathy_debug (DEBUG_DOMAIN, "No pending msg, waiting...");
+		g_signal_connect (tp_chat, "message-received",
+				  G_CALLBACK (status_icon_message_received_cb),
+				  icon);
+		return;
+	}
+
+	status_icon_message_received_cb (tp_chat, messages->data, icon);
+
+	g_list_foreach (messages, (GFunc) g_object_unref, NULL);
+	g_list_free (messages);
+}
+
+static void
+status_icon_message_received_cb (EmpathyTpChat     *tp_chat,
+				 EmpathyMessage    *message,
+				 EmpathyStatusIcon *icon)
+{
+	EmpathyContact  *sender;
+	gchar           *msg;
+	StatusIconEvent *event;
+
+	empathy_debug (DEBUG_DOMAIN, "Message received, add event");
+
+	g_signal_handlers_disconnect_by_func (tp_chat,
+			  		      status_icon_message_received_cb,
+			  		      icon);
+
+	sender = empathy_message_get_sender (message);
+	msg = g_strdup_printf (_("New message from %s:\n%s"),
+			       empathy_contact_get_name (sender),
+			       empathy_message_get_body (message));
+
+	event = status_icon_event_new (icon, EMPATHY_IMAGE_NEW_MESSAGE, msg);
+	event->func = status_icon_event_msg_cb;
+	event->user_data = tp_chat;
+}
+
+static void
+status_icon_idle_notify_cb (EmpathyStatusIcon *icon)
+{
+	EmpathyStatusIconPriv *priv;
+	McPresence             flash_state;
+
+	priv = GET_PRIV (icon);
+
+	flash_state = empathy_idle_get_flash_state (priv->idle);
+	if (flash_state != MC_PRESENCE_UNSET) {
+		const gchar *icon_name;
+
+		icon_name = empathy_icon_name_for_presence_state (flash_state);
+		if (!priv->flash_state_event) {
+			/* We are now flashing */
+			priv->flash_state_event = status_icon_event_new (icon, icon_name, NULL);
+			priv->flash_state_event->user_data = icon;
+			priv->flash_state_event->func = status_icon_event_flash_state_cb;
+		} else {
+			/* We are still flashing but with another state */
+			g_free (priv->flash_state_event->icon_name);
+			priv->flash_state_event->icon_name = g_strdup (icon_name);
+		}
+	}
+	else if (priv->flash_state_event) {
+		/* We are no more flashing */
+		status_icon_event_remove (icon, priv->flash_state_event);
+		priv->flash_state_event = NULL;
+	}
+
+	if (!priv->showing_event_icon) {
 		status_icon_set_from_state (icon);
 	}
 
@@ -274,7 +375,7 @@ status_icon_set_from_state (EmpathyStatusIcon *icon)
 	priv = GET_PRIV (icon);
 
 	state = empathy_idle_get_state (priv->idle);
-	icon_name = gossip_icon_name_for_presence_state (state);
+	icon_name = empathy_icon_name_for_presence_state (state);
 	gtk_status_icon_set_from_icon_name (priv->icon, icon_name);
 }
 
@@ -286,27 +387,27 @@ status_icon_toggle_visibility (EmpathyStatusIcon *icon)
 
 	priv = GET_PRIV (icon);
 
-	visible = gossip_window_get_is_visible (GTK_WINDOW (priv->window));
+	visible = empathy_window_get_is_visible (GTK_WINDOW (priv->window));
 
 	if (visible) {
-		gtk_widget_hide (GTK_WIDGET (priv->window));
-		gossip_conf_set_bool (gossip_conf_get (),
-				      GOSSIP_PREFS_UI_MAIN_WINDOW_HIDDEN, TRUE);
+		empathy_window_iconify (priv->window, priv->icon);
+		empathy_conf_set_bool (empathy_conf_get (),
+				      EMPATHY_PREFS_UI_MAIN_WINDOW_HIDDEN, TRUE);
 	} else {
 		GList *accounts;
 
-		gossip_window_present (GTK_WINDOW (priv->window), TRUE);
-		gossip_conf_set_bool (gossip_conf_get (),
-				      GOSSIP_PREFS_UI_MAIN_WINDOW_HIDDEN, FALSE);
+		empathy_window_present (GTK_WINDOW (priv->window), TRUE);
+		empathy_conf_set_bool (empathy_conf_get (),
+				      EMPATHY_PREFS_UI_MAIN_WINDOW_HIDDEN, FALSE);
 	
 		/* Show the accounts dialog if there is no enabled accounts */
 		accounts = mc_accounts_list_by_enabled (TRUE);
 		if (accounts) {
 			mc_accounts_list_free (accounts);
 		} else {
-			gossip_debug (DEBUG_DOMAIN,
+			empathy_debug (DEBUG_DOMAIN,
 				      "No enabled account, Showing account dialog");
-			gossip_accounts_dialog_show (GTK_WINDOW (priv->window));
+			empathy_accounts_dialog_show (GTK_WINDOW (priv->window));
 		}
 	}
 }
@@ -319,6 +420,9 @@ status_icon_activate_cb (GtkStatusIcon     *status_icon,
 
 	priv = GET_PRIV (icon);
 
+	empathy_debug (DEBUG_DOMAIN, "Activated: %s",
+		       priv->events ? "event" : "toggle");
+
 	if (priv->events) {
 		status_icon_event_remove (icon, priv->events->data);
 	} else {
@@ -348,7 +452,7 @@ status_icon_popup_menu_cb (GtkStatusIcon     *status_icon,
 
 	priv = GET_PRIV (icon);
 
-	show = gossip_window_get_is_visible (GTK_WINDOW (priv->window));
+	show = empathy_window_get_is_visible (GTK_WINDOW (priv->window));
 
 	g_signal_handlers_block_by_func (priv->show_window_item,
 					 status_icon_show_hide_window_cb,
@@ -359,7 +463,7 @@ status_icon_popup_menu_cb (GtkStatusIcon     *status_icon,
 					   status_icon_show_hide_window_cb,
 					   icon);
 
-	submenu = gossip_presence_chooser_create_menu ();
+	submenu = empathy_presence_chooser_create_menu ();
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->status_item),
 				   submenu);
 
@@ -379,7 +483,7 @@ status_icon_create_menu (EmpathyStatusIcon *icon)
 
 	priv = GET_PRIV (icon);
 
-	glade = gossip_glade_get_file ("empathy-status-icon.glade",
+	glade = empathy_glade_get_file ("empathy-status-icon.glade",
 				       "tray_menu",
 				       NULL,
 				       "tray_menu", &priv->popup_menu,
@@ -388,7 +492,7 @@ status_icon_create_menu (EmpathyStatusIcon *icon)
 				       "tray_status", &priv->status_item,
 				       NULL);
 
-	gossip_glade_connect (glade,
+	empathy_glade_connect (glade,
 			      icon,
 			      "tray_new_message", "activate", status_icon_new_message_cb,
 			      "tray_quit", "activate", status_icon_quit_cb,
@@ -409,7 +513,7 @@ status_icon_new_message_cb (GtkWidget         *widget,
 
 	priv = GET_PRIV (icon);
 
-	//gossip_new_message_dialog_show (GTK_WINDOW (priv->window));
+	//empathy_new_message_dialog_show (GTK_WINDOW (priv->window));
 }
 
 static void
@@ -428,7 +532,7 @@ status_icon_show_hide_window_cb (GtkWidget         *widget,
 
 static void
 status_icon_local_pending_cb (EmpathyContactManager *manager,
-			      GossipContact         *contact,
+			      EmpathyContact         *contact,
 			      gchar                 *message,
 			      EmpathyStatusIcon     *icon)
 {
@@ -440,14 +544,14 @@ status_icon_local_pending_cb (EmpathyContactManager *manager,
 	priv = GET_PRIV (icon);
 
 	for (l = priv->events; l; l = l->next) {
-		if (gossip_contact_equal (contact, ((StatusIconEvent*)l->data)->user_data)) {
+		if (empathy_contact_equal (contact, ((StatusIconEvent*)l->data)->user_data)) {
 			return;
 		}
 	}
 
 	str = g_strdup_printf (_("Subscription requested for %s\n"
 				 "Message: %s"),
-			       gossip_contact_get_name (contact),
+			       empathy_contact_get_name (contact),
 			       message);
 
 	event = status_icon_event_new (icon, GTK_STOCK_DIALOG_QUESTION, str);
@@ -460,15 +564,41 @@ status_icon_local_pending_cb (EmpathyContactManager *manager,
 static void
 status_icon_event_subscribe_cb (StatusIconEvent *event)
 {
-	GossipContact *contact;
+	EmpathyContact *contact;
 
-	contact = GOSSIP_CONTACT (event->user_data);
+	contact = EMPATHY_CONTACT (event->user_data);
 
 	empathy_subscription_dialog_show (contact, NULL);
 
 	g_object_unref (contact);
 }
 
+static void
+status_icon_event_flash_state_cb (StatusIconEvent *event)
+{
+	EmpathyStatusIconPriv *priv;
+
+	priv = GET_PRIV (event->user_data);
+
+	empathy_idle_set_flash_state (priv->idle, MC_PRESENCE_UNSET);
+}
+
+static void
+status_icon_event_msg_cb (StatusIconEvent *event)
+{
+	EmpathyFilter *filter;
+	EmpathyTpChat *tp_chat;
+
+	empathy_debug (DEBUG_DOMAIN, "Dispatching text channel");
+
+	tp_chat = event->user_data;
+	filter = g_object_get_data (G_OBJECT (tp_chat), "filter");
+	empathy_filter_process (filter,
+				empathy_tp_chat_get_channel (tp_chat),
+				TRUE);
+	g_object_unref (tp_chat);
+}
+
 static StatusIconEvent *
 status_icon_event_new (EmpathyStatusIcon *icon,
 		       const gchar       *icon_name,
@@ -485,6 +615,7 @@ status_icon_event_new (EmpathyStatusIcon *icon,
 
 	priv->events = g_list_append (priv->events, event);
 	if (!priv->blink_timeout) {
+		priv->showing_event_icon = FALSE;
 		priv->blink_timeout = g_timeout_add (BLINK_TIMEOUT,
 						     (GSourceFunc) status_icon_event_timeout_cb,
 						     icon);
@@ -507,19 +638,17 @@ status_icon_event_remove (EmpathyStatusIcon *icon,
 	}
 	priv->events = g_list_remove (priv->events, event);
 	status_icon_event_free (event);
+	priv->showing_event_icon = FALSE;
 	status_icon_update_tooltip (icon);
+	status_icon_set_from_state (icon);
 
 	if (priv->events) {
 		return;
 	}
 
-	status_icon_set_from_state (icon);
-	priv->showing_state_icon = TRUE;
-
 	if (priv->blink_timeout) {
 		g_source_remove (priv->blink_timeout);
 		priv->blink_timeout = 0;
-
 	}
 }
 
@@ -530,9 +659,9 @@ status_icon_event_timeout_cb (EmpathyStatusIcon *icon)
 
 	priv = GET_PRIV (icon);
 
-	priv->showing_state_icon = !priv->showing_state_icon;
+	priv->showing_event_icon = !priv->showing_event_icon;
 
-	if (priv->showing_state_icon) {
+	if (!priv->showing_event_icon) {
 		status_icon_set_from_state (icon);
 	} else {
 		StatusIconEvent *event;
